;;----------------------------------------------------------------------------
;; SASME.ASM - SASME.DLL (Subspace ASM Encryption)  Coconut emulator, may 2000
;; Compiler: MASM 6.14      Debugger: SoftICE 4.05       Last update: april 2004
;;----------------------------------------------------------------------------
;;
;;                     S U B S P A C E    R O U T I N E S
;;
;; This DLL must be called from Visual Basic only, because it is using the
;; string VB data type.
;;
;; This DLL implements routines intended to be used at client side, in bots.
;; SSGetKey01 is client and billing side and SSGetKey02 is server side only.
;; The rest are mostly server and client side (encryption routines, checksums).
;; The packet encryption routines are client, server and billing side. Finally, 
;; SSBillingPasswordEncrypt is billing side only.
;;
;;----------------------------------------------------------------------------
;; SASME.DLL is using: KERNEL32.DLL, ZLIB.DLL
;;
;;----------------------------------------------------------------------------
;; Visual Basic declares and data types:
;; 
;; Declare Function SSBillingPasswordEncrypt Lib "sasme" Alias "F1" (StrToEncrypt As String) As String
;; Declare Sub SSInitEncrTable Lib "sasme" Alias "F2" (SSEncrTable As SSTable)
;; Declare Function SSEncrypt Lib "sasme" Alias "F3" (SSEncrTable As SSTable, StrToEncr As String, ByVal StartPos As Integer) As Long
;; Declare Function SSDecrypt Lib "sasme" Alias "F4" (SSEncrTable As SSTable, StrToDecr As String, ByVal StartPos As Integer) As Long
;; Declare Sub SSRandomize Lib "sasme" Alias "F5" (ByVal Seed As Long)
;; Declare Function SSRnd Lib "sasme" Alias "F6" () As Integer
;; Declare Function SSGetKey01 Lib "sasme" Alias "F7" () As Long
;; Declare Function SSGetKey02 Lib "sasme" Alias "F8" (ByVal Key01 As Long) As Long
;; Declare Function SSGetPingValue Lib "sasme" Alias "F9" () As Long
;; Declare Function SSGet03PingValue Lib "sasme" Alias "F10" () As Long
;; Declare Sub SSPoke03ChecksumAt0A Lib "sasme" Alias "F11" (Packet03 As String)
;; Declare Function SSInitialLvlChecksum Lib "sasme" Alias "F12" (LvlFile As String) As Long
;; Declare Sub SSParseLvlFile Lib "sasme" Alias "F13" (MapBuffer As Byte, LvlFile As String)
;; Declare Function SSParameterChecksum1A Lib "sasme" Alias "F14" (Settings As String, ByVal Seed As Long) As Long
;; Declare Function SSCodeChecksum1A Lib "sasme" Alias "F15" (SubspaceBin As String, ByVal Seed As Long) As Long [needs subspace.bin]
;; Declare Function SSLevelChecksum1A Lib "sasme" Alias "F16" (MapBuffer As Byte, ByVal Seed As Long) As Long
;; Declare Function SSCodeChecksum1A Lib "sasme" Alias "F17" (ByVal Seed As Long) As Long [does not needs subspace.bin]
;; Type SSTable
;;     SSKey           As Long
;;     SSBuf(1 To 520) As Byte
;; End Type
;;
;;----------------------------------------------------------------------------
;; Visual Basic calls (pseudo code):
;;
;;
;; 1. Billing password encryption.
;;   EncryptedUserPassword$ = SSBillingPasswordEncrypt(UserPassword$)
;;   EncryptedSquadPassword$ = SSBillingPasswordEncrypt(SquadPassword$)
;;
;;
;; 2. Initialize SSTable structure.
;;   Dim SSEncrTable as SSTable
;;   SSEncrTable.SSKey = &H12345678 'server key 02
;;   Call SSInitEncrTable(SSEncrTable)
;;
;;
;; 3. Encrypt packets.
;;   Packet$ = Chr(0) & Chr(5) & ...   'These are 2 examples of packets to encrypt
;;   Packet$ = Chr(3) & Chr(0) & ...   '		''
;;
;;   If Mid(Packet$, 1, 1) <> Chr(0) Then
;;       StartPos% = 2
;;   Else
;;       StartPos% = 3
;;   End If
;;
;;   Select Case SSEncrypt(SSEncrTable, Packet$, StartPos%)
;;       Case 0 : SendTo Packet$
;;       Case 1 : MsgBox "Encryption error 1: StartPos = 0"
;;       Case 2 : MsgBox "Encryption error 2: StartPos > Len(StrToEncr)"
;;       Case 3 : MsgBox "Encryption error 3: NumByToEncr > 520"
;;   End Select
;;
;;
;; 4. Decrypt packets.
;;   RecvFrom Packet$
;;
;;   If Mid(Packet$, 1, 1) <> Chr(0) Then
;;       StartPos% = 2
;;   Else
;;       StartPos% = 3
;;   End If
;;
;;   Select Case SSDecrypt(SSEncrTable, Packet$, StartPos%)
;;       Case 0 : Call PacketHandler(Packet$)
;;       Case 1 : MsgBox "Decryption error 1: StartPos = 0"
;;       Case 2 : MsgBox "Decryption error 2: StartPos > Len(StrToDecr)"
;;       Case 3 : MsgBox "Decryption error 3: NumByToDecr > 520"
;;   End Select
;;
;;
;; 5. Initialize seed for SS rnd generator.
;;   Call SSRandomize(0)      'Randomize timer (It forces the use of GetTickCount as seed)
;;   Call SSRandomize(25)     'Randomize 25
;;   Call SSrandomize(Long1&) 'Randomize to Long1 value
;;
;;
;; 6. Get random number in range 0-32767.
;;   Int1% = SSRnd   'To get an integer random number in range 0-32767
;;
;;
;; 7. Get long key for packet 01.
;;   Packet$ = Chr(0) & Chr(1) & LongToString(SSGetKey01) & Chr(1) & Chr(0))
;;
;;
;; 8. Calculate the 02 key of a given 01 key.
;;	 Key02& = SSGetKey02(Key01&)
;;
;;
;; 9. Ping the server (STREAM mode). Also used in 05 sync packets.
;;   Packet$ = LongToString(SSGetPingValue)
;;
;;
;; 10. Get long value for bytes 2,3,4,5 of 03 position packet.
;;   Packet03$ = Chr(3) & Chr(0) & LongToString(SSGet03PingValue) & ...
;;
;;
;; 11. Get byte value for byte number 0A of 03 position packet.
;;   'The 0A byte is a checksum of the first 16H bytes of 03 position packet,
;;   'so u can only call this routine when u have composed the bytes from 0
;;   'to 15H of the 03 pos. packet.
;;   'It is very important to put 0 at position 0A before calling this.
;;   'The routine will put the value in position 0A of 03 pos. packet, so you only
;;   'have to call the routine and it stores the right value in the right
;;   'position.
;;   Call SSPoke03ChecksumAt0A(Packet03$)
;;
;;
;; 12. Calculate the checksum of a .lvl map file (at login time only).
;;   InitialMapChecksum& = SSInitialLvlChecksum(LvlFile$)
;;
;;
;; 13. Parse a .lvl map file to the format used by the SS graphic engine.
;;   Dim MapBuffer(0 To 1023, 0 To 1023) As Byte
;;   Call SSParseLvlFile(MapBuffer(0, 0), LvlFile$)
;;
;;
;; 14. The security checksum.
;;   'When a 18 packet type is received the client have to return a reliable 
;;   '1A packet containing the following checksums:
;;   Seed& = StringToLong(Right(Packet18$, 4))
;;   If Seed& <> 0 Then
;;       ParameterChecksum& = SSParameterChecksum1A(Settings$, Seed&)
;;       CodeChecksum& = SSCodeChecksum1A(SubspaceBin$, Seed&)
;;       LevelChecksum& = SSLevelChecksum1A(MapBuffer(0, 0), Seed&)
;;       '
;;       SendTo ReliableHeader & Chr(&H1A) & LongToString(0) & _
;;         LongToString(ParameterChecksum&) & _
;;         LongToString(CodeChecksum&) & _
;;         LongToString(LevelChecksum&) & _
;;         String(23, 0)
;;   End If
;;
;;----------------------------------------------------------------------------
.386

.MODEL flat,stdcall

OPTION CASEMAP:NONE
option prologue:none
option epilogue:none
option dotname

Include windows.inc
Include kernel32.inc
IncludeLib kernel32.lib

;-----------------------------------------------------------------------------
.DataSeg	segment

	;This data segment is going to be unique and shared by all programs
	;which are using the DLL at same time.
	;This is done with the "-section:.DataSeg,rws" directive for the linker.

align DWORD

	;Arrays used by SSCodeChecksum1A (F15) routine
	BlockLength		dd		0D5A0h, 2480h, 6F70h, 21E0h, 0D3C0h, 5FE0h, 1C90h, 8BE0h, 0E0h
	BlockBase		dd		6940h, 400h, 1CAE0h, 42E70h, 305A0h, 2A5B0h, 3960h, 13EF0h, 5750h

	;Thanks to Catid for suplying this huge dictionary.
	;Array (data dictionary) used by SSInitialLvlChecksum (F12) routine
	OneTimePad		dd		000000000h, 077073096h, 0ee0e612ch, 0990951bah,
							0076dc419h, 0706af48fh, 0e963a535h, 09e6495a3h,
							00edb8832h, 079dcb8a4h, 0e0d5e91eh, 097d2d988h,
							009b64c2bh, 07eb17cbdh, 0e7b82d07h, 090bf1d91h,
							01db71064h, 06ab020f2h, 0f3b97148h, 084be41deh,
							01adad47dh, 06ddde4ebh, 0f4d4b551h, 083d385c7h,
							0136c9856h, 0646ba8c0h, 0fd62f97ah, 08a65c9ech,
							014015c4fh, 063066cd9h, 0fa0f3d63h, 08d080df5h
					dd		03b6e20c8h, 04c69105eh, 0d56041e4h, 0a2677172h,
							03c03e4d1h, 04b04d447h, 0d20d85fdh, 0a50ab56bh,
							035b5a8fah, 042b2986ch, 0dbbbc9d6h, 0acbcf940h,
							032d86ce3h, 045df5c75h, 0dcd60dcfh, 0abd13d59h,
							026d930ach, 051de003ah, 0c8d75180h, 0bfd06116h,
							021b4f4b5h, 056b3c423h, 0cfba9599h, 0b8bda50fh,
							02802b89eh, 05f058808h, 0c60cd9b2h, 0b10be924h,
							02f6f7c87h, 058684c11h, 0c1611dabh, 0b6662d3dh
					dd		076dc4190h, 001db7106h, 098d220bch, 0efd5102ah,
							071b18589h, 006b6b51fh, 09fbfe4a5h, 0e8b8d433h,
							07807c9a2h, 00f00f934h, 09609a88eh, 0e10e9818h,
							07f6a0dbbh, 0086d3d2dh, 091646c97h, 0e6635c01h,
							06b6b51f4h, 01c6c6162h, 0856530d8h, 0f262004eh,
							06c0695edh, 01b01a57bh, 08208f4c1h, 0f50fc457h,
							065b0d9c6h, 012b7e950h, 08bbeb8eah, 0fcb9887ch,
							062dd1ddfh, 015da2d49h, 08cd37cf3h, 0fbd44c65h
					dd		04db26158h, 03ab551ceh, 0a3bc0074h, 0d4bb30e2h,
							04adfa541h, 03dd895d7h, 0a4d1c46dh, 0d3d6f4fbh,
							04369e96ah, 0346ed9fch, 0ad678846h, 0da60b8d0h,
							044042d73h, 033031de5h, 0aa0a4c5fh, 0dd0d7cc9h,
							05005713ch, 0270241aah, 0be0b1010h, 0c90c2086h,
							05768b525h, 0206f85b3h, 0b966d409h, 0ce61e49fh,
							05edef90eh, 029d9c998h, 0b0d09822h, 0c7d7a8b4h,
							059b33d17h, 02eb40d81h, 0b7bd5c3bh, 0c0ba6cadh,
							0edb88320h, 09abfb3b6h, 003b6e20ch, 074b1d29ah
					dd		0ead54739h, 09dd277afh, 004db2615h, 073dc1683h,
							0e3630b12h, 094643b84h, 00d6d6a3eh, 07a6a5aa8h,
							0e40ecf0bh, 09309ff9dh, 00a00ae27h, 07d079eb1h,
							0f00f9344h, 08708a3d2h, 01e01f268h, 06906c2feh,
							0f762575dh, 0806567cbh, 0196c3671h, 06e6b06e7h,
							0fed41b76h, 089d32be0h, 010da7a5ah, 067dd4acch,
							0f9b9df6fh, 08ebeeff9h, 017b7be43h, 060b08ed5h,
							0d6d6a3e8h, 0a1d1937eh, 038d8c2c4h, 04fdff252h
					dd		0d1bb67f1h, 0a6bc5767h, 03fb506ddh, 048b2364bh,
							0d80d2bdah, 0af0a1b4ch, 036034af6h, 041047a60h,
							0df60efc3h, 0a867df55h, 0316e8eefh, 04669be79h,
							0cb61b38ch, 0bc66831ah, 0256fd2a0h, 05268e236h,
							0cc0c7795h, 0bb0b4703h, 0220216b9h, 05505262fh,
							0c5ba3bbeh, 0b2bd0b28h, 02bb45a92h, 05cb36a04h,
							0c2d7ffa7h, 0b5d0cf31h, 02cd99e8bh, 05bdeae1dh,
							09b64c2b0h, 0ec63f226h, 0756aa39ch, 0026d930ah
					dd		09c0906a9h, 0eb0e363fh, 072076785h, 005005713h,
							095bf4a82h, 0e2b87a14h, 07bb12baeh, 00cb61b38h,
							092d28e9bh, 0e5d5be0dh, 07cdcefb7h, 00bdbdf21h,
							086d3d2d4h, 0f1d4e242h, 068ddb3f8h, 01fda836eh,
							081be16cdh, 0f6b9265bh, 06fb077e1h, 018b74777h,
							088085ae6h, 0ff0f6a70h, 066063bcah, 011010b5ch,
							08f659effh, 0f862ae69h, 0616bffd3h, 0166ccf45h,
							0a00ae278h, 0d70dd2eeh, 04e048354h, 03903b3c2h
					dd		0a7672661h, 0d06016f7h, 04969474dh, 03e6e77dbh,
							0aed16a4ah, 0d9d65adch, 040df0b66h, 037d83bf0h,
							0a9bcae53h, 0debb9ec5h, 047b2cf7fh, 030b5ffe9h,
							0bdbdf21ch, 0cabac28ah, 053b39330h, 024b4a3a6h,
							0bad03605h, 0cdd70693h, 054de5729h, 023d967bfh,
							0b3667a2eh, 0c4614ab8h, 05d681b02h, 02a6f2b94h,
							0b40bbe37h, 0c30c8ea1h, 05a05df1bh, 02d02ef8dh

	OTP				equ		OFFSET FLAT:OneTimePad

.DataSeg	ends
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
.DataSeg2	segment

	;This data segment is going to be private for each proccess which is
	;using the DLL. It means the system loader will load a separate copy
	;of this segment for each process which uses the DLL.

align DWORD

	LenDestString	dd		?			;Data used by SSBillingPasswordEncrypt (F1) routine
	DestString		db		33 dup (0)	;		''

align DWORD

	BufAux			db		528 dup(0)	;Buffer used by encr/decr (F3/F4) routines

	RndSeed			dd		0			;Seed for random generator SSRnd (F6) routine

	PtrBlockLength	dd		?			;Data used by SSCodeChecksum1A (F15) routine
	PtrBlockBase	dd		?			;		''
	Counter			dd		?			;		''

	AuxDWord		dd		?			;Data used by SSLevelChecksum1A (F16) routine
	LevelChecksum	dd		?			;		''
    
    ClientChecksum  dd      ?           ;Data used by SSCodeChecksum1A (F17) routine   
    PartSum         dd      ?           ;       '' 
      
.DataSeg2	ends
;-----------------------------------------------------------------------------


.code

align DWORD
DllEntry proc hInstance:HINSTANCE, reason:DWORD, reserved1:DWORD
	;
	; If u want to define a language (english, spanish, etc) for the DLL,
	; or perform some initialization process when the DLL is loaded,
	; this is the place to do it. The 2 lines coded in this routine are
	; the minimum that this routine must do, just return a good response
	; to the caller (the caller is the system program loader), indicating
	; the DLL was successfully loaded.
	;
	mov  eax,TRUE
	ret
DllEntry Endp
;-----------------------------------------------------------------------------
F1		proc	stdcall public, StrToEncryptPtr:DWORD
	;
	;SSBillingPasswordEncrypt
	;(Billing)
	;
	;Encrypt a string (max length is 32 chars) using the billing password
	;encryption algorithm (one way encryption)
	;
	push	ecx					;Save registers
	push	edx					;  ''
	push	esi					;  ''
	push	edi					;  ''
	push	ebp					;  ''
	nrp	= 5*4					;nrp = extra offset of params on stack
	;
	;Validation of the length of StrToEncrypt, it must be in range 1-32
	mov				esi, [esp+nrp+4]	;esi = ptr to StrToEncrypt
	mov				esi, [esi]			;	''
	.if				(esi == 0)			;If ptr to StrToEncrypt = 0 Then return empty string
		xor			eax, eax			;	''
		jmp			ExitFunction		;Exit Function
	.endif
	mov				ebp, [esi-4]		;ebp = Len(StrToEncrypt)
	.if				(ebp > 32)			;If Len(StrToEncryptPtr) > 32 Then return empty string
		xor			eax, eax			;	''
		jmp			ExitFunction		;Exit Function
	.endif
	;
	;Save ptr to StrToEncrypt and Len(StrToEncrypt) on stack
	push			esi					;Save ptr to StrToEncrypt (esi) on stack
	push			ebp					;Save length of StrToEncrypt (ebp) on stack
	;
	;Seed loop (xor all bytes of StrToEncrypt among them to calculate a factor
	;in eax to be used as seed for encryption)
	xor				eax, eax			;eax = 0 (initialize seed for encryption)
	.repeat								;Repeat (calculate seed value in eax)
		mov			dl, [esi]			;  dl = next char of StrToEncrypt
		.if			(dl == 0)			;  Exit this loop if a 0 char is found in StrToEncrypt
			.break						;		''
		.endif							;		''
		and			eax, 0ffh			;  Truncate eax to 0ffh bit mask
		and			edx, 0ffh			;  Truncate edx to 0ffh bit mask
		xor			eax, edx			;  eax = eax xor edx
		inc			esi					;  esi = esi + 1
		dec			ebp					;  ebp = ebp - 1
	.until			(ebp == 0)			;Until ebp = 0
	;
	;Restore ptr to StrToEncrypt and Len(StrToEncrypt) from stack
	pop				ebp					;Restore length of StrToEncrypt (ebp) from stack
	pop				esi					;Restore ptr to StrToEncrypt (esi) from stack
	;
	;Point edi to DestString buffer and initialize LenDestString
	mov				edi, OFFSET FLAT:DestString		;edi = ptr to DestString
	mov				LenDestString, 0				;Initialize LenDestString
	;
	;Encryption loop
	.repeat								;Repeat (encrypt StrToEncrypt in DestString buffer)
		mov			cl, [esi]			;  cl = next char of StrToEncrypt
		.if			(cl == 0)			;  Exit this loop if a 0 char is found in StrToEncrypt
			.break						;		''
		.endif							;		''
		xor			cl, al				;  xor char of StrToEncrypt with the seed (encrypt char)
		.if			(cl != 0)			;  If encrypted char <> 0
			mov		[edi], cl			;    Poke encrypted char in DestString
		.else							;  Else
			mov		byte ptr[edi], 0edh	;    Poke 0edh in DestString
		.endif							;  Endif
		movsx		edx, cl				;  Seed = (Seed xor (EncryptedChar * (2^(EncryptedChar And 3)))) And 255
		mov			ecx, edx			;		''
		and			ecx, 3				;		''
		shl			edx, cl				;		''
		xor			eax, edx			;		''
		inc			esi					;  Point esi to next char of StrToEncrypt
		inc			edi					;  Point edi to next char of DestString
		inc			LenDestString		;  Poke the length of DestString in LenDestString
		dec			ebp					;  ebp = ebp - 1
	.until			(ebp == 0)			;Until ebp = 0
	;
	;Check LenDestString value
	.if				(LenDestString == 0)		;If Len(DestString) = 0 Then
		xor			eax, eax					;  Return empty string
	.else										;Else
		mov			byte ptr [edi], 0			;  Poke 0 at end of DestString
		mov			eax, OFFSET FLAT:DestString	;  Return DestString
	.endif
	;
	ExitFunction:
	;
	pop		ebp					;Restore registers
	pop		edi					;  ''
	pop		esi					;  ''
	pop		edx					;  ''
	pop		ecx					;  ''
	;
	ret		4
	;
	;SSBillingPasswordEncrypt
	;
F1		endp
;-----------------------------------------------------------------------------

align DWORD

;-----------------------------------------------------------------------------
F2		proc	stdcall public, SSTablePtr:DWORD
	;
	;SSInitEncrTable
	;(Client, Server, Billing)
	;
	;Initialize SSTable.SSBuf byte array, using dword SSTable.SSKey as seed
	;(packet encryption)
	;
	push	eax					;Save registers
	push	ebx					;  ''
	push	ecx					;  ''
	push	edx					;  ''
	push	esi					;  ''
	push	edi					;  ''
	nrp	= 6*4					;nrp = extra offset of params on stack
	;
	;Set ptr's and get values into SSTable structure
	mov		esi, [esp+nrp+4]	;esi = SSTablePtr (ptr to SSTable struct)
	lea		edi, [esi+4]		;edi = ptr to SSTable.SSBuf
	mov		ebx, [esi]			;ebx = value of SSTable.SSKey
	;
	;Set loop counter value
	mov		ecx, 104h			;ecx = LoopCounter (SSBuf size=208h by)
	;
	.repeat
		mov			eax, 834E0B5Fh			;Calculations
		imul		ebx						;   ''
		add			edx, ebx				;   ''
		sar			edx, 10h				;   ''
		mov			eax, edx				;   ''
		shr			eax, 1Fh				;   ''
		add			edx, eax				;   ''
		lea			eax, [edx+8*edx]		;   ''
		shl			eax, 3					;   ''
		sub			eax, edx				;   ''
		lea			eax, [eax+4*eax]		;   ''
		shl			eax, 1					;   ''
		sub			eax, edx				;   ''
		shl			eax, 2					;   ''
		mov			esi, eax				;   ''
		mov			eax, ebx				;   ''
		cdq									;   ''
		mov			ebx, 1F31Dh				;   ''
		idiv		ebx						;   ''
		imul		edx, 41A7h				;   ''
		sub			edx, esi				;   ''
		add			edx, 7Bh				;   ''
		mov			ebx, edx				;   ''
		cmp			ebx, 0					;	''
		jg			endif1					;	''
			lea		ebx, [edx+7FFFFFFFh]	;   ''
		endif1:								;	''
		mov			[edi], bx				;Save 2 by to SSBuf
		add			edi, 2					;Point to next position in SSBuf
	.untilcxz								;ecx=ecx-1, loop until ecx=0
	;
	pop		edi					;Restore registers
	pop		esi					;  ''
	pop		edx					;  ''
	pop		ecx					;  ''
	pop		ebx					;  ''
	pop		eax					;  ''
	;
	ret		4					;Exit
	;
	;SSInitEncrTable
	;
F2		endp
;-----------------------------------------------------------------------------

align DWORD

;-----------------------------------------------------------------------------
F3		proc  stdcall public, SSTablePtr:DWORD, StrPtr:DWORD, StartPos:DWORD
	;
	;SSEncrypt
	;(Client, Server, Billing)
	;
	;Encrypt packet
	;
	push	ebx					;Save registers
	push	ecx					;  ''
	push	edx					;  ''
	push	esi					;  ''
	push	edi					;  ''
	push	ebp					;  ''
	nrp	= 6*4					;nrp = extra offset of params on stack
	;
	;Validate for error 1, StartPos = 0
	mov		eax, [esp+nrp+12]	;Load StartPos in eax
	cmp		eax, 0				;If StartPos = 0 Then Exit Sub
	je		SSEncr_Err1			;  ''
	;
	;Validate for error 2, StartPos > Len(Str)
	mov		esi, [esp+nrp+8]	;esi = StrPtr (ptr to Str to encrypt)
	mov		esi, [esi]			;Point esi to Str
	mov		ecx, [esi-4]		;Load Len(Str) in ecx
	cmp		eax, ecx			;If StartPos > Len(Str) Then Exit Sub
	ja		SSEncr_Err2			;  ''
	;
	;Validate for error 3, NumByToEncr > 520
	dec		eax					;StartPos = StartPos - 1
	sub		ecx, eax			;ecx=NumByToEncr=Len(Str)-(StartPos-1)
	cmp		ecx, 520			;If NumByToEncr > 520 Then Exit Sub
	ja		SSEncr_Err3			;  ''
	;
	;Copy Str to encrypt (from StartPos to EndOfString) to BufAux
	add		esi, eax			;Point esi to StartPos into Str
	lea		edi, [BufAux]		;Point edi to BufAux
	push	ecx					;Save ecx (ecx = NumByToEncr)
	cld							;clear direction flag (to make movsb to go upward)
	rep		movsb				;Copy the string from memory to memory
	pop		ecx					;Restore ecx
	;
	;Set ptr's and get values into SSTable structure
	mov		edx, [esp+nrp+4]	;edx = SSTablePtr (ptr to SSTable struct)
	lea		edi, [edx+4]		;edi = ptr to SSTable.SSBuf
	mov		ebp, [edx]			;ebp = value of SSTable.SSKey
	;
	;Point ebx to BufAux
	lea		ebx, [BufAux]		;ebx = ptr to BufAux
	;
	;Set loop counter value
	push	ecx					;Save ecx
	shr		ecx, 2				;ecx = LoopCounter = (NumByToEncr/4)+1
	inc		ecx					;  ''
	;
	;Encryption loop
	.repeat
		mov		esi, [edi]		;Load 4 by of SSBuf in esi
		xor		esi, ebp		;SSBuf xor SSKey
		mov		ebp, [ebx]		;Load 4 by to encrypt (of BufAux) in ebp
		xor		ebp, esi		;4 by to encr xor (SSBuf xor SSKey)
		mov		[ebx], ebp		;Save 4 by encrypted to BufAux (and use them as next SSKey)
		add		edi, 4			;Point to next position in SSBuf
		add		ebx, 4			;Point to next position in BufAux
	.untilcxz					;ecx=ecx-1, loop until ecx=0
	;
	;Copy encrypted BufAux to Str (from StartPos to EndOfString)
	pop		ecx					;Restore ecx (ecx = NumByToEncr)
	mov		edi, [esp+nrp+8]	;edi = StrPtr (ptr to Str to encrypt)
	mov		edi, [edi]			;Point edi to Str
	add		edi, eax			;Point edi to StartPos into Str

	lea		esi, [BufAux]		;esi = ptr to BufAux
	rep		movsb				;Copy the string from memory to memory
	;
	xor		eax, eax			;Return value in eax: 0 = successful
	;
SSEncr_End:
	pop		ebp					;Restore registers
	pop		edi					;  ''
	pop		esi					;  ''
	pop		edx					;  ''
	pop		ecx					;  ''
	pop		ebx					;  ''
	;
	ret		3*4					;Exit
	;
SSEncr_Err1:
	mov		eax, 1				;Error 1: StartPos = 0
	jmp		SSEncr_End			;  ''
	;
SSEncr_Err2:
	mov		eax, 2				;Error 2: StartPos > Len(Str)
	jmp		SSEncr_End			;  ''
	;
SSEncr_Err3:
	mov		eax, 3				;Error 3: NumByToEncr > 520
	jmp		SSEncr_End			;  ''
	;
	;SSEncrypt
	;
F3		endp
;-----------------------------------------------------------------------------

align DWORD

;-----------------------------------------------------------------------------
F4		proc  stdcall public, SSTablePtr:DWORD, StrPtr:DWORD, StartPos:DWORD
	;
	;SSDecrypt
	;(Client, Server, Billing)
	;
	;Decrypt packet
	;
	push	ebx					;Save registers
	push	ecx					;  ''
	push	edx					;  ''
	push	esi					;  ''
	push	edi					;  ''
	push	ebp					;  ''
	nrp	= 6*4					;nrp = extra offset of params on stack
	;
	;Validate for error 1, StartPos = 0
	mov		eax, [esp+nrp+12]	;Load StartPos in eax
	cmp		eax, 0				;If StartPos = 0 Then Exit Sub
	je		SSDecr_Err1			;  ''
	;
	;Validate for error 2, StartPos > Len(Str)
	mov		esi, [esp+nrp+8]	;esi = StrPtr (ptr to Str to decrypt)
	mov		esi, [esi]			;Point esi to Str
	mov		ecx, [esi-4]		;Load Len(Str) in ecx
	cmp		eax, ecx			;If StartPos > Len(Str) Then Exit Sub
	ja		SSDecr_Err2			;  ''
	;
	;Validate for error 3, NumByToDecr > 520
	dec		eax					;StartPos = StartPos - 1
	sub		ecx, eax			;ecx=NumByToDecr=Len(Str)-(StartPos-1)
	cmp		ecx, 520			;If NumByToDecr > 520 Then Exit Sub
	ja		SSDecr_Err3			;  ''
	;
	;Copy Str to decrypt (from StartPos to EndOfString) to BufAux
	add		esi, eax			;Point esi to StartPos into Str
	lea		edi, [BufAux]		;Point edi to BufAux
	push	ecx					;Save ecx (ecx = NumByToDecr)
	cld							;clear direction flag (to make movsb to go upward)
	rep		movsb				;Copy the string from memory to memory
	pop		ecx					;Restore ecx
	;
	;Set ptr's and get values into SSTable structure
	mov		edx, [esp+nrp+4]	;edx = SSTablePtr (ptr to SSTable struct)
	lea		edi, [edx+4]		;edi = ptr to SSTable.SSBuf
	mov		ebp, [edx]			;ebp = value of SSTable.SSKey
	;
	;Point ebx to BufAux
	lea		ebx, [BufAux]		;ebx = ptr to BufAux
	;
	;Set loop counter value
	push	ecx					;Save ecx
	shr		ecx, 2				;ecx = LoopCounter = (NumByToDecr/4)+1
	inc		ecx					;  ''
	;
	;Decryption loop
	.repeat
		mov		esi, [edi]		;Load 4 by of SSBuf in esi
		mov		edx, [ebx]		;Load 4 by to decr (of BufAux) in edx
		xor		esi, ebp		;SSBuf xor SSKey
		xor		esi, edx		;(SSBuf xor SSKey) xor 4 by to decr
		mov		[ebx], esi		;Save 4 by decrypted in BufAux
		mov		ebp, edx		;Use 4 by to decr as next SSKey
		add		edi, 4			;Point to next position in SSBuf
		add		ebx, 4			;Point to next position in BufAux
	.untilcxz					;ecx=ecx-1, loop until ecx=0
	;
	;Copy decrypted BufAux to Str (from StartPos to EndOfString)
	pop		ecx					;Restore ecx (ecx = NumByToDecr)
	mov		edi, [esp+nrp+8]	;edi = StrPtr (ptr to Str to decrypt)
	mov		edi, [edi]			;Point edi to Str
	add		edi, eax			;Point edi to StartPos into Str
	lea		esi, [BufAux]		;esi = ptr to BufAux
	rep		movsb				;Copy the string from memory to memory
	;
	xor		eax, eax			;Return value in eax: 0 = successful
	;
SSDecr_End:
	pop		ebp					;Restore registers
	pop		edi					;  ''
	pop		esi					;  ''
	pop		edx					;  ''
	pop		ecx					;  ''
	pop		ebx					;  ''
	;
	ret		3*4					;Exit
	;
SSDecr_Err1:
	mov		eax, 1				;Error 1: StartPos = 0
	jmp		SSDecr_End			;  ''
	;
SSDecr_Err2:
	mov		eax, 2				;Error 2: StartPos > Len(Str)
	jmp		SSDecr_End			;  ''
	;
SSDecr_Err3:
	mov		eax, 3				;Error 3: NumByToDecr > 520
	jmp		SSDecr_End			;  ''
	;
	;SSDecrypt
	;
F4		endp
;-----------------------------------------------------------------------------

align DWORD

;-----------------------------------------------------------------------------
F5		proc	stdcall public, Seed:DWORD
	;
	;SSRandomize
	;(Client, Server)
	;
	;Initialize seed value for SSRnd random engine
	;
	push	eax					;Save registers
	push	edx					;  ''
	nrp	= 2*4					;nrp = extra offset of params on stack
	;
	mov			eax, [esp+nrp+4]		;Load Seed value in eax
	.if			(eax == 0)				;Check for eax not to be 0
		invoke GetTickCount					;If eax = 0 Then eax = GetTickCount
	.endif
	mov			RndSeed, eax			;Store eax in RndSeed
	;
	pop		edx					;Restore registers
	pop		eax					;  ''
	;
	ret		4					;Exit
	;
	;SSRandomize
	;
F5		endp
;-----------------------------------------------------------------------------

align DWORD

;-----------------------------------------------------------------------------
F6		proc	stdcall public
	;
	;SSRnd
	;(Client, Server)
	;
	;Random engine. Compute a random number (range 0-7FFFh)
	;
	push	ecx					;Save registers
	push	edx					;  ''
	;
	mov			eax, RndSeed			;Load Seed value in eax
	.if			(eax == 0)				;Check for eax not to be 0
		invoke GetTickCount					;If eax = 0 Then eax = GetTickCount
	.endif
	;
	;Compute pseudo-random number
	lea		ecx, [eax+2*eax]			;Calculations
	lea		edx, [eax+4*ecx]			;  ''
	shl		edx, 4						;  ''
	add		edx, eax					;  ''
	shl		edx, 8						;  ''
	sub		edx, eax					;  ''
	lea		eax, [eax+4*edx+269EC3h]	;  ''
	mov		RndSeed, eax				;  '' (Store seed for next number)
	sar		eax, 10						;  ''
	and		eax, 7FFFh					;  '' (ax = return value (0-7FFFh))
	;
	;Return pseudo-random number in ax/eax (range 0-7FFFh)
	;
	pop		edx					;Restore registers
	pop		ecx					;  ''
	;
	ret							;Exit
	;
	;SSRnd
	;
F6		endp
;-----------------------------------------------------------------------------

align DWORD

;-----------------------------------------------------------------------------
F7		proc	stdcall public
	;
	;SSGetKey01
	;(Client, Billing)
	;
	;Compute dword value (to be used as key in packet 01 to server)
	;
	push	ecx					;Save registers
	push	edx					;  ''
	push	esi					;  ''
	push	edi					;  ''
	;
	invoke GetTickCount				;eax = GetTickCount
	mov		ecx, eax			;Calculations
	mov		eax, 0CCCCCCCDh		;  ''
	mul		ecx					;  ''
	mov		edi, edx			;  ''
	shr		edi, 3				;  ''
	call	F6					;  '' (Get random number 0-7FFF in eax)  (F6 = SSRnd)
	mov		esi, eax			;  ''
	shl		esi, 10h			;  ''
	call	F6					;  '' (Get random number 0-7FFF in eax)  (F6 = SSRnd)
	add		edi, esi			;  ''
	add		eax, edi			;  ''
	cdq							;  ''
	xor		eax, edx			;  ''
	sub		eax, edx			;  ''
	neg		eax					;  ''
	;
	;Return 01 key in eax
	;
	pop		edi					;Restore registers
	pop		esi					;  ''
	pop		edx					;  ''
	pop		ecx					;  ''
	;
	ret							;Exit
	;
	;SSGetKey01
	;
F7		endp
;-----------------------------------------------------------------------------

align DWORD

;-----------------------------------------------------------------------------
F8		proc	stdcall public, Key01:DWORD
	;
	;SSGetKey02
	;(Server)
	;
	;Compute the 02 key of a given 01 key (both dword)
	;
	mov		eax, [esp+4]		;Load Key01 value in eax
	;
	xor		eax, 0ffffffffh		;Calculate key02
	inc		eax					;	''
	;
	;Return 02 key in eax
	;
	ret		4					;Exit
	;
	;SSGetKey02
	;
F8		endp
;-----------------------------------------------------------------------------

align DWORD

;-----------------------------------------------------------------------------
F9		proc	stdcall public
	;
	;SSGetPingValue
	;(Client, Server, Billing)
	;
	;Compute dword value (to ping the SS server with it, also used in 05 sync
	;packets)
	;
	push	edx					;Save registers
	;
	invoke GetTickCount				;eax = GetTickCount
	mov		edx, eax			;Calculations
	mov		eax, 0CCCCCCCDh		;  ''
	mul		edx					;  ''
	shr		edx, 3				;  ''
	mov		eax, edx			;  '' (move return value to eax)
	;
	;Return value in eax
	;
	pop		edx					;Restore registers
	;
	ret							;Exit
	;
	;SSGetPingValue
	;
F9		endp
;-----------------------------------------------------------------------------

align DWORD

;-----------------------------------------------------------------------------
F10		proc	stdcall public
	;
	;SSGet03PingValue (old SSProc1)
	;(Client)
	;
	;Compute dword value (to be used in positions 2,3,4,5 of 03 pos. packet)
	;
	push	edx					;Save registers
	;
	invoke GetTickCount				;eax = GetTickCount
	mov		edx, eax			;Calculations
	mov		eax, 0CCCCCCCDh		;  ''
	mul		edx					;  ''
	shr		edx, 3				;  ''
	add		edx, eax			;  ''
	and		edx, 7FFFFFFFh		;  ''
	;
	;Return value in eax
	mov		eax, edx			;eax = return value
	;
	pop		edx					;Restore registers
	;
	ret							;Exit
	;
	;SSGet03PingValue (old SSProc1)
	;
F10		endp
;-----------------------------------------------------------------------------

align DWORD

;-----------------------------------------------------------------------------
F11		proc	stdcall public, Packet03Ptr:DWORD
	;
	;SSPoke03ChecksumAt0A (old SSProc2)
	;(Client)
	;
	;Compute and poke byte value (poking it in position 0A of 03 pos. packet)
	;
	push	eax					;Save registers
	push	ebx					;  ''
	push	ecx					;  ''
	push	edx					;  ''
	nrp	= 4*4					;nrp = extra offset of params on stack
	;
	;Initialize registers
	mov			edx, [esp+nrp+4]	;edx = Ptr to Packet03
	mov			edx, [edx]			; ''
	lea			ebx, [edx+0ah]		;ebx = Ptr to byte 0ah of Packet03
	mov			ecx, 16h			;ecx = LoopCounter (16h)
	xor			al, al				;al = 0
	;
	;Checksum loop
	.repeat
		xor		al, [edx]			;al = al xor byte of Packet03
		inc		edx					;Point edx to next byte of Packet03
	.untilcxz						;ecx=ecx-1, loop until ecx=0
	;
	;Poke checksum in 0A byte
	mov			[ebx], al			;Packet03[0A] = al
	;
	pop		edx					;Restore registers
	pop		ecx					;  ''
	pop		ebx					;  ''
	pop		eax					;  ''
	;
	ret		4					;Exit
	;
	;SSPoke03ChecksumAt0A (old SSProc2)
	;
F11		endp
;-----------------------------------------------------------------------------

align DWORD

;-----------------------------------------------------------------------------
F12		proc	stdcall public, StrLvlFilePtr:DWORD
	;
	;SSInitialLvlChecksum
	;(CLient, Server)
	;
	;Compute dword value (to be used as initial .lvl file checksum)
	;
	push	ebx					;Save registers
	push	ecx					;	''
	push	edx					;	''
	push	esi					;	''
	nrp	= 4*4					;nrp = extra offset of params on stack
	;
	;Initialize registers
	mov		ecx, [esp+nrp+4]	;ecx = address of the first (lower) byte of StrLvlFile
	mov		ecx, [ecx]			;	''
	mov		edx, [ecx-4]		;edx = Len(StrLvlFile)
	lea		esi, [ecx+edx]		;esi = (address of the last (higher) byte of StrLvlFile) + 1
	mov		eax, -1				;eax = 0ffffffffh
	;
	;Calculate initial lvl checksum
	.while		(ecx < esi)			;While ecx < esi
		mov		edx, eax			;  edx = eax
		xor		ebx, ebx			;  ebx = 0
		mov		bl, [ecx]			;  bl = byte of StrLvlFile pointed by ecx
		and		edx, 0ffh			;  Truncate edx to 0ffh bit mask
		xor		edx, ebx			;  edx = edx xor ebx
		sar		eax, 8				;  Shift-arithmetic-right eax's bits 8 times
		mov		edx, [edx*4+OTP]	;  edx = DWORD OneTimePad(edx)
		and		eax, 0ffffffh		;  Truncate eax to 0ffffffh bit mask
		xor		eax, edx			;  eax = eax xor edx
		inc		ecx					;  ecx = ecx + 1
	.endw							;End While
	;
	not		eax					;Return eax value
	;
	pop		esi					;Restore registers
	pop		edx					;	''
	pop		ecx					;	''
	pop		ebx					;	''
	;
	ret		4					;Exit
	;
	;SSInitialLvlChecksum
	;
F12		endp
;-----------------------------------------------------------------------------

align DWORD

;-----------------------------------------------------------------------------
F13		proc	stdcall public, MapBufferPtr:DWORD, StrLvlFilePtr:DWORD
	;
	;SSParseLvlFile
	;(Client, Server)
	;
	;Parse a .lvl map file to the format used by the SS graphic engine
	;
	;Structure of .lvl files:
	;1.	The .lvl file may have a tileset or not. Levels starting with "BM"
	;	contains a tileset. Level files which does not start with "BM"
	;	doesn't have a custom tileset, they are using the default SS tileset.
	;	The tileset is just a .bmp file (that is why it starts with "BM").
	;	It is a 304x160 pixels/256 colors bitmap. Each tile is 16x16 pixels,
	;	so the tileset is 19x10 tiles.
	;	The bmp size (thus the tileset size) is stored in a DWORD right after the
	;	"BM" signature, at offset 2.
	;	This size must be used to skip the tileset and find the starting address
	;	of the map definition into the .lvl file.
	;2.	After the tileset comes the map definition. Level files which does not
	;	have a tileset starts with the map definition. 
	;	The map definition contains one DWORD for each tile in the map.
	;	All data relative to a single tile is stored into a DWORD variable.
	;	That DWORD contains the position (row, column) into the map and the tile type.
	;	These 3 variables are stored using the following bit mask:
	;		Column...12	bits (low)
	;		Row......12	bits (medium)
	;		Type.....8 bits  (high)
	;	Row/Column values (hex):
	;		0 to 3FF (The parsed map size is 1024x1024 tiles)
	;	Tile type values (hex):
	;		00		 = Space
	;		AA		 = Flag
	;		01 to BE = Tiles from tileset
	;		DB		 = Station
	;		DC		 = Wormhole
	;		D8 to DA = Asteroids
	;
	;I wrote this routine looking at the following C++ code written by Catid.
	;My thanks goes to him for saving me the time of extracting it with SoftICE.
	;	void ParseMapFile(char * FileContents, LONG Length)
	;	{
	;	    for (int i = 0; i < Length; i += 4)
	;	    {
	;	        LONG value = *(LONG*)&FileContents[i];
	;	        Map[(value >> 0x0C & 0xFFF)][(value & 0xFFF)] = value >> 0x18;
	;	    }
	;	}
	;
	push	eax					;Save registers
	push	ebx					;	''
	push	ecx					;	''
	push	edx					;	''
	push	esi					;	''
	push	edi					;	''
	nrp	= 6*4					;nrp = extra offset of params on stack
	;
	;Initialize registers
	mov		esi, [esp+nrp+8]	;esi = Ptr to StrLvlFile
	mov		esi, [esi]			;	''
	mov		edx, [esi-4]		;edx = max address of StrLvlFile
	lea		edx, [edx+esi-1]	;	''
	mov		edi, [esp+nrp+4]	;edi = Ptr to MapBuffer
	;
	.if			(word ptr [esi] == "MB")	;yes, no mistake, it is "MB", because it is a word, so byte order is reversed (word "MB" = string "BM")
		add		esi, [esi+2]	;Skipping the tileset
	.endif
	;
	;Lvl parse loop
	.while		(esi < edx)		;While esi < edx
		mov		eax, [esi]		;  Load next DWORD of the .lvl file in eax
		mov		ebx, eax		;  Load row in ebx
		shr		ebx, 0ch		;		''
		and		ebx, 0fffh		;		''
		shl		ebx, 0ah		;  row = row * 400h
		mov		ecx, eax		;  Load column in ecx
		and		ecx, 0fffh		;		''
		add		ebx, ecx		;  Use row, column to point into MapBuffer
		shr		eax, 18h		;  Load the tile type in eax (al)
		mov		[edi+ebx], al	;  Poke the tile type into MapBuffer
		add		esi, 4			;  Point esi to next DWORD of the .lvl file
	.endw						;End While
	;
	pop		edi					;Restore registers
	pop		esi					;	''
	pop		edx					;	''
	pop		ecx					;	''
	pop		ebx					;	''
	pop		eax					;	''
	;
	ret		2*4					;Exit
	;
	;SSParseLvlFile
	;
F13		endp
;-----------------------------------------------------------------------------

align DWORD

;-----------------------------------------------------------------------------
F14		proc	stdcall public, StrSettingsPtr:DWORD, Seed:DWORD
	;
	;SSParameterChecksum1A
	;(Client, Server)
	;
	;Compute dword value (to be used as Parameter checksum in the 1A response
	;to security checksum packet 18)
	;
	push	ecx					;Save registers
	push	edx					;  ''
	push	esi					;  ''
	push	edi					;  ''
	nrp	= 4*4					;nrp = extra offset of params on stack
	;
	;Initialize registers
	mov		eax, [esp+nrp+4]	;eax = Ptr to StrSettings
	mov		eax, [eax]			;   ''
	mov		esi, [esp+nrp+8]	;esi = Seed value
	mov		ecx, 165h			;ecx = Counter
	xor		edi, edi			;edi = 0 (edi will contain the parameter checksum)
	;
	;Parameter checksum loop
	.repeat
		mov		edx, [eax]		;Load next DWORD of settings buffer in edx
		add		eax, 4			;Point to next DWORD into settings buffer
		xor		edx, esi		;Xor edx with the seed
		add		edi, edx		;Add edx to edi
	.untilcxz					;ecx=ecx-1, loop until ecx=0
	;
	mov		eax, edi			;Return value = edi
	;
	pop		edi					;Restore registers
	pop		esi					;  ''
	pop		edx					;  ''
	pop		ecx					;  ''
	;
	ret		2*4					;Exit
	;
	;SSParameterChecksum1A
	;
F14		endp

;-----------------------------------------------------------------------------

align DWORD

;-----------------------------------------------------------------------------
F15		proc	stdcall public, StrSubspaceBinPtr:DWORD, Seed:DWORD
	;
	;SSCodeChecksum1A 
	;(Client, Server)
	;
	;Compute dword value (to be used as Code checksum in the 1A response
	;to security checksum packet 18) uses subspace.bin
	;
	push	ebx					;Save registers
	push	ecx					;  ''
	push	edx					;  ''
	push	esi					;  ''
	push	edi					;  ''
	push	ebp					;  ''
	nrp	= 6*4					;nrp = extra offset of params on stack
	;
	;Initialize registers
	mov		eax, [esp+nrp+4]	;ebp = Ptr to StrSubspaceBin
	mov		ebp, [eax]			;   ''
	mov		eax, [esp+nrp+8]	;eax = Seed value
	xor		edi, edi			;edi = 0 (edi will contain the code checksum)
	;
	;Code checksum loop
	mov					PtrBlockLength, OFFSET FLAT:BlockLength
	mov					PtrBlockBase, OFFSET FLAT:BlockBase
	mov					Counter, 0
	.repeat
		xor				esi, esi				;Initialize partial acumulator
		mov				ecx, PtrBlockLength		;Load BlockLength ocurrence in ecx
		mov				ecx, [ecx]				;      ''
		mov				edx, PtrBlockBase		;Load BlockBase ocurrence in edx
		mov				edx, [edx]				;      ''
		add				edx, ebp				;      ''
		.repeat
			mov			ebx, [edx]				;Load next dword of subspace.bin buffer in ebx
			test		ecx, 7ffh
			jz			else1
				xor		ebx, esi				;Do the checksum
				xor		ebx, eax				;     ''
				mov		esi, ebx				;     ''
				jmp		endif1
			else1:
				or		ebx, eax				;Do the checksum
				add		esi, ebx				;     ''
			endif1:
			sub			ecx, 3					;Calculate new value for counter
			add			edx, 3					;Point to next dword into subspace.bin buffer
		.until			(ecx <= 4)
		add				edi, esi				;Add partial checksum to edi
		add				PtrBlockLength, 4		;Point to next BlockLength ocurrence
		add				PtrBlockBase, 4			;Point to next BlockBase ocurrence
		inc				Counter					;Calculate new value for counter
	.until				(Counter == 9)
	;
	mov		eax, edi			;Return value = edi
	;
	pop		ebp					;Restore registers
	pop		edi					;  ''
	pop		esi					;  ''
	pop		edx					;  ''
	pop		ecx					;  ''
	pop		ebx					;  ''
	;
	ret		2*4					;Exit
	;
	;SSCodeChecksum1A
	;
F15		endp
;-----------------------------------------------------------------------------

align DWORD

;-----------------------------------------------------------------------------
F16		proc	stdcall public, MapBufferPtr:DWORD, Seed:DWORD
	;
	;SSLevelChecksum1A
	;(Client, Server)
	;
	;Compute dword value (to be used as Level checksum in the 1A response
	;to security checksum packet 18)
	;
	push	ebx					;Save registers
	push	ecx					;  ''
	push	edx					;  ''
	push	esi					;  ''
	push	edi					;  ''
	push	ebp					;  ''
	nrp	= 6*4					;nrp = extra offset of params on stack
	;
	;Calculate level checksum (sorry for the lack of comments, this is too sloppy)
	mov		edi, [esp+nrp+8]	;edi = Seed value
	;
	mov		LevelChecksum, edi	;Do some initial validation
	mov		eax, edi			;	''
	cdq							;	''
	xor		eax, edx			;	''
	sub		eax, edx			;	''
	and		eax, 1fh			;	''
	xor		eax, edx			;	''
	mov		ecx, eax			;	''
	sub		ecx, edx			;	''
	cmp		ecx, 400h			;	''
	jge		Exit_LevelCheck		;	''
	;
	mov		eax, edi			;Do some initialization
	cdq							;	''
	mov		esi, 1fh			;	''
	idiv	esi					;	''
	mov		eax, [esp+nrp+4]	;	'' (eax = Ptr to MapBuffer)
	mov		esi, ecx			;	''
	shl		esi, 0ah			;	''
	add		esi, eax			;	''
	mov		eax, 400h			;	''
	mov		ebx, 41fh			;	''
	sub		ebx, ecx			;	''
	sub		eax, edx			;	''
	add		esi, edx			;	''
	mov		AuxDWord, eax		;	''
	shr		ebx, 5				;	''
	;
	;Level checksum loop
	.repeat								;Do the checksum
		lea			edx, [esi+eax]		;	''
		mov			ecx, esi			;	''
		cmp			esi, edx			;	''
		jae			skip1				;	''
		.repeat							;	''
			 mov	al, [ecx]			;	''
			 test	al, al				;	''
			 jbe	else1				;	''
			 cmp	al, 0a1h			;	''
			 jb		else2				;	''
		   else1:						;	''
			 cmp	al, 0abh			;	''
			 jnz	else3				;	''
		   else2:						;	''
			 and	eax, 0ffh			;	''
			 mov	ebp, edi			;	''
			 xor	ebp, eax			;	''
			 mov	eax, LevelChecksum	;	''
			 add	eax, ebp			;	''
			 mov	LevelChecksum, eax	;	''
		   else3:						;	''
			 add	ecx, 1fh			;	''
		.until		(ecx >= edx)		;	''
		mov			eax, AuxDWord		;	''
		skip1:							;	''
		add			esi, 8000h			;	''
		dec			ebx					;	''
	.until			(ebx == 0)			;	''
	;
	Exit_LevelCheck:
	;
	mov		eax, LevelChecksum	;Return LevelChecksum value in eax
	;
	pop		ebp					;Restore registers
	pop		edi					;  ''
	pop		esi					;  ''
	pop		edx					;  ''
	pop		ecx					;  ''
	pop		ebx					;  ''
	;
	ret		2*4					;Exit
	;
	;SSLevelChecksum1A
	;
F16		endp
;-----------------------------------------------------------------------------

align DWORD

;-----------------------------------------------------------------------------
F17		proc	stdcall public, Seed:DWORD
    ;
    ;Made by Catid
	;
	;SSCodeChecksum1A 
	;(Client, Server)
	;
	;Compute dword value (to be used as Code checksum in the 1A response
	;to security checksum packet 18) doesn't use subspace.bin
	;
   
        
        mov ClientChecksum, 0
        mov PartSum, 0
         
        ;First Part Calculations
        
        mov PartSum, 0c98ed41fh
       
        mov eax, 3e1bh
        or eax, Seed
        add PartSum, eax
       
        mov eax, 42435942h
        xor eax, Seed
        xor PartSum, eax
       
        mov eax, 1d895300h
        or eax, Seed
        add PartSum, eax
        
        mov eax, 6b5c4032h
        xor eax, Seed
        xor PartSum, eax        
        
        mov eax, 467e44h
        or eax, Seed
        add PartSum, eax

        mov eax, 516c7edah
        xor eax, Seed
        xor PartSum, eax  
        
        mov eax, 8b0c708bh
        or eax, Seed
        add PartSum, eax
        
        mov eax, 6b3e3429h
        xor eax, Seed
        xor PartSum, eax
       
        mov eax, 560674c9h
        or eax, Seed
        add PartSum, eax
                
        mov eax, 0f4e6b721h
        xor eax, Seed
        xor PartSum, eax
               
        mov eax, 0e90cc483h
        or eax, Seed
        add PartSum, eax
                        
        mov eax, 80ece15ah
        xor eax, Seed
        xor PartSum, eax
                       
        mov eax, 728bce33h
        or eax, Seed
        add PartSum, eax
                                
        mov eax, 1fc5d1e6h
        xor eax, Seed
        xor PartSum, eax
                               
        mov eax, 8b0c518bh
        or eax, Seed
        add PartSum, eax
                                        
        mov eax, 24f1a96eh
        xor eax, Seed
        xor PartSum, eax
                                       
        mov eax, 30ae0c1h
        or eax, Seed
        add PartSum, eax
                                                
        mov eax, 8858741bh
        xor eax, Seed
        xor PartSum, eax         
        
        mov eax, PartSum
        add ClientChecksum, eax
        
        ;Second Part Calculations
        
        mov PartSum, 9c15857dh
        
        mov eax, 424448bh
        or eax, Seed
        add PartSum, eax
        
        mov eax, 0cd0455eeh
        xor eax, Seed
        xor PartSum, eax         
        
        mov eax, 727h
        or eax, Seed
        add PartSum, eax        
        
        mov eax, 8d7f29cdh
        xor eax, Seed
        xor PartSum, eax          

        mov eax, PartSum
        add ClientChecksum, eax  
        
        ;Third Part Calculations      
        
        mov PartSum, 824b9278h
        
        mov eax, 6590h
        or eax, Seed
        add PartSum, eax
                
        mov eax, 8e16169ah
        xor eax, Seed
        xor PartSum, eax
        
        mov eax, 8b524914h
        or eax, Seed
        add PartSum, eax
                
        mov eax, 82dce03ah
        xor eax, Seed
        xor PartSum, eax
                
        mov eax, 0fa83d733h
        or eax, Seed
        add PartSum, eax
                        
        mov eax, 0b0955349h
        xor eax, Seed
        xor PartSum, eax        
                
        mov eax, 0e8000003h
        or eax, Seed
        add PartSum, eax
                                
        mov eax, 7cfe3604h
        xor eax, Seed
        xor PartSum, eax                    
         
        mov eax, PartSum
        add ClientChecksum, eax         
        
        ;Fourth Part Calculations
        
        mov PartSum, 0e3f8d2afh
                        
        mov eax, 2de85024h
        or eax, Seed
        add PartSum, eax
                                        
        mov eax, 0bed0296bh
        xor eax, Seed
        xor PartSum, eax
                                
        mov eax, 587501f8h
        or eax, Seed
        add PartSum, eax
                                                
        mov eax, 0ada70f65h
        xor eax, Seed
        xor PartSum, eax
                 
        mov eax, PartSum
        add ClientChecksum, eax
        
        ;Fifth Part Calculations
        
        mov PartSum, 0cb54d8a0h
                                        
        mov eax, 0f000001h
        or eax, Seed
        add PartSum, eax
                                                        
        mov eax, 330f19ffh
        xor eax, Seed
        xor PartSum, eax
                                        
        mov eax, 909090c3h
        or eax, Seed
        add PartSum, eax   
                                                        
        mov eax, 0d20f9f9fh
        xor eax, Seed
        xor PartSum, eax
                                                
        mov eax, 53004addh
        or eax, Seed
        add PartSum, eax 
                                                                
        mov eax, 5d81256bh
        xor eax, Seed
        xor PartSum, eax
                                                        
        mov eax, 8b004b65h
        or eax, Seed
        add PartSum, eax
                                                                        
        mov eax, 0a5312749h
        xor eax, Seed
        xor PartSum, eax
                                                                
        mov eax, 0b8004b67h
        or eax, Seed
        add PartSum, eax
                                                                                
        mov eax, 8adf8fb1h
        xor eax, Seed
        xor PartSum, eax
                                                                        
        mov eax, 8901e283h
        or eax, Seed
        add PartSum, eax
                                                                                        
        mov eax, 8ec94507h
        xor eax, Seed
        xor PartSum, eax
                                                                                
        mov eax, 89d23300h
        or eax, Seed
        add PartSum, eax
                                                                                        
        mov eax, 1ff8e1dch
        xor eax, Seed
        xor PartSum, eax        
                                                                                
        mov eax, 108a004ah
        or eax, Seed
        add PartSum, eax        
                                                                                                
        mov eax, 0c73d6304h
        xor eax, Seed
        xor PartSum, eax   
                                                                                        
        mov eax, 43d2d3h
        or eax, Seed
        add PartSum, eax 
                                                                                                        
        mov eax, 6f78e4ffh
        xor eax, Seed
        xor PartSum, eax  
                         
        mov eax, PartSum
        add ClientChecksum, eax
        
        ;Sixth Part Calculations
        
        mov PartSum, 0b229a3d0h
                                                                                                
        mov eax, 47d708h
        or eax, Seed
        add PartSum, eax
                                                                                                                
        mov eax, 10b0a91h
        xor eax, Seed
        xor PartSum, eax 
                                 
        mov eax, PartSum
        add ClientChecksum, eax
        
        ;Seventh Part Calculations
        
        mov PartSum, 466e55a7h
                                                                                                        
        mov eax, 0c7880d8bh
        or eax, Seed
        add PartSum, eax
                                                                                                                        
        mov eax, 44ce7067h
        xor eax, Seed
        xor PartSum, eax   
                                                                                                                
        mov eax, 0e4h
        or eax, Seed
        add PartSum, eax
                                                                                                                                
        mov eax, 923a6d44h
        xor eax, Seed
        xor PartSum, eax  
                                                                                                                        
        mov eax, 640047d6h
        or eax, Seed
        add PartSum, eax
                                                                                                                                        
        mov eax, 0a62d606ch
        xor eax, Seed
        xor PartSum, eax  
                                                                                                                                
        mov eax, 2bd1f7aeh
        or eax, Seed
        add PartSum, eax
                                                                                                                                                
        mov eax, 2f5621fbh
        xor eax, Seed
        xor PartSum, eax  
                                                                                                                                        
        mov eax, 8b0f74ffh
        or eax, Seed
        add PartSum, eax
                                                                                                                                                        
        mov eax, 2928b332h
        xor eax, Seed
        xor PartSum, eax  
                                         
        mov eax, PartSum
        add ClientChecksum, eax
        
        ;Eighth Part Calculations
        
        mov PartSum, 62cf369ah
                                                 
        mov eax, PartSum
        add ClientChecksum, eax
        
        ;Return Value
        
        mov eax, ClientChecksum        

        ret 4
  
        
        
          
F17	endp



End DllEntry